import struct
import memorymap
import espulp

class StructMemory:
    def __init__(self, address, typecode):
        self.address = address
        self.typecode = typecode
        self.buf = bytearray(struct.calcsize(typecode))
        self.end = address + len(self.buf)

    def __get__(
        self,
        obj,
        objtype = None,
    ):
        self.buf = obj.memory_map[self.address:self.end]
        return struct.unpack_from(self.typecode, self.buf)[0]

    def __set__(self, obj, value) -> None:
        struct.pack_into(self.typecode, self.buf, 0, value)
        obj.memory_map[self.address:self.end] = self.buf

class Blink:
    delay: int = StructMemory(0x000000e8, "I")
    """The delay between toggles in milliseconds"""
    def __init__(self,
led_pin: microcontroller.Pin,
sx_version: int = 3,
delay: int = 500
) -> None:
        self.ulp = espulp.ULP()
        self.ulp.halt()
        self.pins = []
        self.program = bytearray(1)
        self.program[0:244] = (
            b"\x6f\x00\x80\x04\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00" # 00000000
            b"\x82\x80\xa1\x67\x93\x87\x47\x10\x98\x43\xb7\x06\xc0\xfd\xfd\x16" # 00000010
            b"\x75\x8f\x98\xc3\x82\x80\xa1\x67\x93\x87\x47\x10\x37\xc7\x0f\x00" # 00000020
            b"\x98\xc3\x98\x43\xb7\x06\x00\x02\x55\x8f\x98\xc3\x98\x43\xb7\x06" # 00000030
            b"\x40\x00\x55\x8f\x98\xc3\x82\x80\x37\x21\x00\x00\xd9\x37\x11\x20" # 00000040
            b"\xd9\x3f\x83\x27\xc0\x0e\x0d\x47\x63\x97\xe7\x06\x37\x07\x00\x80" # 00000050
            b"\xb5\x67\x23\xa2\xe7\x90\x11\x67\x13\x07\xc7\x45\x83\x27\x00\x0f" # 00000060
            b"\xa9\x66\x93\x85\x46\x48\x8a\x07\xae\x97\xb7\x05\x08\x00\x8c\xc3" # 00000070
            b"\x83\x25\x00\x0f\x85\x47\x93\x86\x06\x41\xa9\x05\xb3\x97\xb7\x00" # 00000080
            b"\x9c\xc2\x05\x45\x85\x46\x29\x68\x83\x27\x00\x0f\x93\x05\x48\x40" # 00000090
            b"\xa9\x07\xb3\x17\xf5\x00\x99\xe2\x93\x05\x88\x40\x9c\xc1\xf3\x25" # 000000a0
            b"\x00\xc0\x83\x27\x80\x0e\xb3\x87\xe7\x02\xae\x97\x63\xe2\xf5\x02" # 000000b0
            b"\x93\xc6\x16\x00\xd1\xbf\x89\x46\x01\x47\xe3\x91\xd7\xfa\x37\x07" # 000000c0
            b"\x00\x80\xb5\x67\x23\xa2\xe7\x94\x09\x67\x13\x07\x47\x13\x79\xb7" # 000000d0
            b"\xf3\x25\x00\xc0\xe1\xbf\x00\x00\xf4\x01\x00\x00\x03\x00\x00\x00" # 000000e0
            b"\x0f\x00\x00\x00" # 000000f0
        )
        self.pins.append(led_pin)
        struct.pack_into("I", self.program, 0x000000f0, espulp.get_rtc_gpio_number(led_pin))
        struct.pack_into("I", self.program, 0x000000ec, sx_version)
        struct.pack_into("I", self.program, 0x000000e8, delay)
        self.memory_map = memorymap.AddressRange(start=0x50000000, length=0x2000)
        self.ulp.run(self.program, pins=self.pins)

