import struct
import memorymap
import espulp

class StructMemory:
    def __init__(self, address, typecode):
        self.address = address
        self.typecode = typecode
        self.buf = bytearray(struct.calcsize(typecode))
        self.end = address + len(self.buf)

    def __get__(
        self,
        obj,
        objtype = None,
    ):
        self.buf = obj.memory_map[self.address:self.end]
        return struct.unpack_from(self.typecode, self.buf)[0]

    def __set__(self, obj, value) -> None:
        struct.pack_into(self.typecode, self.buf, 0, value)
        obj.memory_map[self.address:self.end] = self.buf

class SleepPinWait:
    def __init__(self,
button_pin: microcontroller.Pin,
sx_version: int = 3
) -> None:
        self.ulp = espulp.ULP()
        self.ulp.halt()
        self.pins = []
        self.program = bytearray(1)
        self.program[0:252] = (
            b"\x6f\x00\x80\x04\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00" # 00000000
            b"\x82\x80\xa1\x67\x93\x87\x47\x10\x98\x43\xb7\x06\xc0\xfd\xfd\x16" # 00000010
            b"\x75\x8f\x98\xc3\x82\x80\xa1\x67\x93\x87\x47\x10\x37\xc7\x0f\x00" # 00000020
            b"\x98\xc3\x98\x43\xb7\x06\x00\x02\x55\x8f\x98\xc3\x98\x43\xb7\x06" # 00000030
            b"\x40\x00\x55\x8f\x98\xc3\x82\x80\x37\x21\x00\x00\xd9\x37\x11\x20" # 00000040
            b"\xd9\x3f\x83\x27\x40\x0f\x0d\x47\x63\x92\xe7\x08\xb5\x67\x37\x07" # 00000050
            b"\x00\x80\x23\xa2\xe7\x90\x03\x27\x80\x0f\xa1\x66\x93\x86\x86\x0d" # 00000060
            b"\x8c\x42\x85\x47\x33\x97\xe7\x00\x13\x47\xf7\xff\x6d\x8f\x98\xc2" # 00000070
            b"\x03\x27\x80\x0f\xa9\x65\x13\x85\x45\x48\x0a\x07\x2a\x97\x37\x25" # 00000080
            b"\x08\x08\x08\xc3\x03\x27\x80\x0f\x93\x85\x45\x41\x05\x45\x29\x07" # 00000090
            b"\x33\x97\xe7\x00\x98\xc1\x83\x25\x80\x0f\x98\x42\x29\x68\xb3\x97" # 000000a0
            b"\xb7\x00\xd9\x8f\x9c\xc2\x01\x47\xa1\x65\x83\x26\x80\x0f\xb3\x16" # 000000b0
            b"\xd5\x00\x93\x07\x48\x42\x9c\x43\xa9\x83\xf5\x8f\x8d\xe3\x75\xfb" # 000000c0
            b"\x9c\x4d\x05\x47\x93\xe7\x17\x00\x9c\xcd\xc5\xb7\x09\x47\xe3\x94" # 000000d0
            b"\xe7\xf8\xb5\x67\x37\x07\x00\x80\x23\xa2\xe7\x94\xad\xbf\x01\x47" # 000000e0
            b"\xc9\xbf\x00\x00\x03\x00\x00\x00\x0e\x00\x00\x00" # 000000f0
        )
        self.pins.append(button_pin)
        struct.pack_into("I", self.program, 0x000000f8, espulp.get_rtc_gpio_number(button_pin))
        struct.pack_into("I", self.program, 0x000000f4, sx_version)
        self.memory_map = memorymap.AddressRange(start=0x50000000, length=0x2000)
        self.ulp.run(self.program, pins=self.pins)

